@@ -2,22 +2,23 @@
 /*
  *  Copyright © 2012, 2013 Igalia S.L.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2, or (at your option)
- *  any later version.
+ *  This file is part of Epiphany.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  Epiphany is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  Epiphany is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *  along with Epiphany.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef EPHY_ABOUT_HANDLER_H
-#define EPHY_ABOUT_HANDLER_H
+#pragma once
 
 #include "ephy-history-service.h"
 
@@ -39,5 +40,3 @@ void              ephy_about_handler_handle_request (EphyAboutHandler       *han
 EphyHistoryQuery *ephy_history_query_new_for_overview (void);
 
 G_END_DECLS
-
-#endif /* EPHY_ABOUT_HANDLER_H */
