@@ -567,6 +567,9 @@ ephy_embed_prefs_init (gpointer user_data)
                                                        "enable-developer-extras", TRUE,
                                                        "enable-dns-prefetching", TRUE,
                                                        "enable-plugins", FALSE,
+                                                       "enable-media-stream", TRUE,
+                                                       "enable-smooth-scrolling", TRUE,
+                                                       "javascript-can-open-windows-automatically", TRUE,
                                                        NULL);
 
   for (i = 0; i < G_N_ELEMENTS (webkit_pref_entries); i++) {
@@ -598,10 +601,6 @@ ephy_embed_prefs_init (gpointer user_data)
                    EPHY_PREFS_WEB_DEFAULT_ENCODING,
                    webkit_settings, DEFAULT_ENCODING_SETTING,
                    G_SETTINGS_BIND_GET);
-  g_settings_bind (EPHY_SETTINGS_WEB,
-                   EPHY_PREFS_WEB_ENABLE_SMOOTH_SCROLLING,
-                   webkit_settings, "enable-smooth-scrolling",
-                   G_SETTINGS_BIND_GET);
   g_settings_bind (EPHY_SETTINGS_WEB,
                    EPHY_PREFS_WEB_ENABLE_SITE_SPECIFIC_QUIRKS,
                    webkit_settings, "enable-site-specific-quirks",
