@@ -29,16 +29,17 @@
 
 #include <gdk/gdkkeysyms.h>
 #include <glib/gi18n.h>
-#include <gtk/gtk.h>
+#include <handy.h>
 #include <string.h>
 #include <webkit2/webkit2.h>
 
 struct _EphyFindToolbar {
-  HdySearchBar parent_instance;
+  GtkBin parent_instance;
 
   GCancellable *cancellable;
   WebKitWebView *web_view;
   WebKitFindController *controller;
+  GtkWidget *search_bar;
   GdTaggedEntry *entry;
   GdTaggedEntryTag *entry_tag;
   GtkWidget *next;
@@ -52,7 +53,7 @@ struct _EphyFindToolbar {
   gboolean typing_ahead;
 };
 
-G_DEFINE_TYPE (EphyFindToolbar, ephy_find_toolbar, HDY_TYPE_SEARCH_BAR)
+G_DEFINE_TYPE (EphyFindToolbar, ephy_find_toolbar, GTK_TYPE_BIN)
 
 enum {
   PROP_0,
@@ -362,7 +363,7 @@ ephy_find_toolbar_load_changed_cb (WebKitWebView   *web_view,
                                    EphyFindToolbar *toolbar)
 {
   if (load_event == WEBKIT_LOAD_STARTED &&
-      hdy_search_bar_get_search_mode (HDY_SEARCH_BAR (toolbar))) {
+      hdy_search_bar_get_search_mode (HDY_SEARCH_BAR (toolbar->search_bar))) {
     ephy_find_toolbar_close (toolbar);
   }
 }
@@ -372,20 +373,17 @@ ephy_find_toolbar_init (EphyFindToolbar *toolbar)
 {
   GtkWidget *clamp;
   GtkWidget *box;
-  GtkSizeGroup *size_group;
 
-  size_group = gtk_size_group_new (GTK_SIZE_GROUP_VERTICAL);
+  toolbar->search_bar = hdy_search_bar_new ();
+  gtk_container_add (GTK_CONTAINER (toolbar), toolbar->search_bar);
 
   clamp = GTK_WIDGET (hdy_clamp_new ());
   hdy_clamp_set_maximum_size (HDY_CLAMP (clamp), 400);
   hdy_clamp_set_tightening_threshold (HDY_CLAMP (clamp), 300);
-  gtk_container_add (GTK_CONTAINER (toolbar), clamp);
+  gtk_container_add (GTK_CONTAINER (toolbar->search_bar), clamp);
 
   box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
-  gtk_style_context_add_class (gtk_widget_get_style_context (box),
-                               GTK_STYLE_CLASS_RAISED);
-  gtk_style_context_add_class (gtk_widget_get_style_context (box),
-                               GTK_STYLE_CLASS_LINKED);
+  gtk_style_context_add_class (gtk_widget_get_style_context (box), "linked");
   gtk_container_add (GTK_CONTAINER (clamp), box);
 
   toolbar->entry = gd_tagged_entry_new ();
@@ -394,7 +392,6 @@ ephy_find_toolbar_init (EphyFindToolbar *toolbar)
   gd_tagged_entry_tag_set_has_close_button (toolbar->entry_tag, FALSE);
 
   gtk_widget_set_hexpand (GTK_WIDGET (toolbar->entry), TRUE);
-  gtk_entry_set_max_length (GTK_ENTRY (toolbar->entry), 512);
   gtk_entry_set_placeholder_text (GTK_ENTRY (toolbar->entry), _("Type to search…"));
   gtk_container_add (GTK_CONTAINER (box), GTK_WIDGET (toolbar->entry));
 
@@ -403,7 +400,6 @@ ephy_find_toolbar_init (EphyFindToolbar *toolbar)
   gtk_widget_set_tooltip_text (toolbar->prev,
                                _("Find previous occurrence of the search string"));
   gtk_container_add (GTK_CONTAINER (box), toolbar->prev);
-  gtk_widget_show_all (toolbar->prev);
   gtk_widget_set_sensitive (toolbar->prev, FALSE);
 
   /* Next */
@@ -413,11 +409,6 @@ ephy_find_toolbar_init (EphyFindToolbar *toolbar)
   gtk_container_add (GTK_CONTAINER (box), toolbar->next);
   gtk_widget_set_sensitive (toolbar->next, FALSE);
 
-  gtk_size_group_add_widget (size_group, GTK_WIDGET (toolbar->entry));
-  gtk_size_group_add_widget (size_group, toolbar->next);
-  gtk_size_group_add_widget (size_group, toolbar->prev);
-  g_object_unref (size_group);
-
   /* connect signals */
   g_signal_connect (toolbar->entry, "icon-release",
                     G_CALLBACK (search_entry_clear_cb), toolbar);
@@ -431,7 +422,7 @@ ephy_find_toolbar_init (EphyFindToolbar *toolbar)
                             G_CALLBACK (ephy_find_toolbar_find_next), toolbar);
   g_signal_connect_swapped (toolbar->prev, "clicked",
                             G_CALLBACK (ephy_find_toolbar_find_previous), toolbar);
-  hdy_search_bar_connect_entry (HDY_SEARCH_BAR (toolbar),
+  hdy_search_bar_connect_entry (HDY_SEARCH_BAR (toolbar->search_bar),
                                 GTK_ENTRY (toolbar->entry));
 
   search_entry_changed_cb (GTK_ENTRY (toolbar->entry), toolbar);
@@ -645,10 +636,12 @@ ephy_find_toolbar_selection_async (GObject      *source_object,
 
     str_value = jsc_value_to_string (value);
     exception = jsc_context_get_exception (jsc_value_get_context (value));
-    if (exception)
+    if (exception) {
       g_warning ("Error running javascript: %s", jsc_exception_get_message (exception));
-    else if (strlen (str_value))
+    } else if (strlen (str_value)) {
       gtk_entry_set_text (GTK_ENTRY (toolbar->entry), str_value);
+      gtk_editable_select_region (GTK_EDITABLE (toolbar->entry), 0, -1);
+    }
   }
 }
 
@@ -666,15 +659,15 @@ ephy_find_toolbar_open (EphyFindToolbar *toolbar,
 
   gtk_editable_select_region (GTK_EDITABLE (toolbar->entry), 0, -1);
 
-  hdy_search_bar_set_search_mode (HDY_SEARCH_BAR (toolbar), TRUE);
-  hdy_search_bar_set_show_close_button (HDY_SEARCH_BAR (toolbar), TRUE);
+  hdy_search_bar_set_search_mode (HDY_SEARCH_BAR (toolbar->search_bar), TRUE);
+  hdy_search_bar_set_show_close_button (HDY_SEARCH_BAR (toolbar->search_bar), TRUE);
   gtk_widget_grab_focus (GTK_WIDGET (toolbar->entry));
 }
 
 void
 ephy_find_toolbar_close (EphyFindToolbar *toolbar)
 {
-  hdy_search_bar_set_search_mode (HDY_SEARCH_BAR (toolbar), FALSE);
+  hdy_search_bar_set_search_mode (HDY_SEARCH_BAR (toolbar->search_bar), FALSE);
 
   if (toolbar->web_view == NULL) return;
 
@@ -684,7 +677,7 @@ ephy_find_toolbar_close (EphyFindToolbar *toolbar)
 void
 ephy_find_toolbar_request_close (EphyFindToolbar *toolbar)
 {
-  if (hdy_search_bar_get_search_mode (HDY_SEARCH_BAR (toolbar))) {
+  if (hdy_search_bar_get_search_mode (HDY_SEARCH_BAR (toolbar->search_bar))) {
     g_signal_emit (toolbar, signals[CLOSE], 0);
   }
 }
