@@ -70,6 +70,7 @@ typedef enum {
   EPHY_WEB_VIEW_ERROR_PAGE_NETWORK_ERROR,
   EPHY_WEB_VIEW_ERROR_PAGE_CRASH,
   EPHY_WEB_VIEW_ERROR_PROCESS_CRASH,
+  EPHY_WEB_VIEW_ERROR_UNRESPONSIVE_PROCESS,
   EPHY_WEB_VIEW_ERROR_INVALID_TLS_CERTIFICATE,
   EPHY_WEB_VIEW_ERROR_UNSAFE_BROWSING,
   EPHY_WEB_VIEW_ERROR_NO_SUCH_FILE,
@@ -104,6 +105,7 @@ gboolean            ephy_web_view_get_should_bypass_safe_browsing (EphyWebView
 void                ephy_web_view_set_should_bypass_safe_browsing (EphyWebView               *view,
                                                                    gboolean                   bypass_safe_browsing);
 gboolean                   ephy_web_view_get_is_blank             (EphyWebView               *view);
+gboolean                   ephy_web_view_is_newtab                (EphyWebView               *view);
 gboolean                   ephy_web_view_is_overview              (EphyWebView               *view);
 void                       ephy_web_view_has_modified_forms       (EphyWebView               *view,
                                                                    GCancellable              *cancellable,
@@ -182,4 +184,8 @@ void                       ephy_web_view_show_auth_form_save_request (EphyWebVie
                                                                       gpointer                        response_data,
                                                                       GDestroyNotify                  response_destroy);
 
+GtkWidget                 *ephy_web_view_new_with_user_content_manager (WebKitUserContentManager *ucm);
+
+guint64                    ephy_web_view_get_uid                       (EphyWebView *web_view);
+
 G_END_DECLS
