@@ -1,5 +1,7 @@
 'use strict';
 
+/* global Ephy */
+
 Ephy.Overview = class Overview
 {
     constructor(model)
@@ -26,15 +28,15 @@ Ephy.Overview = class Overview
 
     _initialize()
     {
-        let anchors = document.getElementsByTagName('a');
+        const anchors = document.getElementsByTagName('a');
         for (let i = 0; i < anchors.length; i++) {
-            let anchor = anchors[i];
-            if (anchor.className != 'overview-item')
+            const anchor = anchors[i];
+            if (anchor.className !== 'overview-item')
                 continue;
 
-            let item = new Ephy.Overview.Item(anchor);
+            const item = new Ephy.Overview.Item(anchor);
 
-            let closeButton = anchor.getElementsByClassName('overview-close-button')[0];
+            const closeButton = anchor.getElementsByClassName('overview-close-button')[0];
             closeButton.onclick = (event) => {
                 this._removeItem(anchor);
                 event.preventDefault();
@@ -42,7 +44,7 @@ Ephy.Overview = class Overview
 
             // URLs and titles are always sent from the UI process, but thumbnails
             // aren't, so update the model with the thumbnail if there's one.
-            let thumbnailPath = item.thumbnailPath();
+            const thumbnailPath = item.thumbnailPath();
             if (thumbnailPath)
                 this._model.setThumbnail(item.url(), thumbnailPath);
             else
@@ -51,15 +53,15 @@ Ephy.Overview = class Overview
             this._items.push(item);
         }
 
-        let items = this._model.urls;
+        const items = this._model.urls;
         if (items.length > this._items.length)
             this._onURLsChanged(items);
 
-        for (let thumbnailChange of this._pendingThumbnailChanges)
+        for (const thumbnailChange of this._pendingThumbnailChanges)
             this._onThumbnailChanged(thumbnailChange.url, thumbnailChange.path);
         this._pendingThumbnailChanges = [];
 
-        for (let titleChange of this._pendingTitleChanges)
+        for (const titleChange of this._pendingTitleChanges)
             this._onTitleChanged(titleChange.url, titleChange.title);
         this._pendingTitleChanges = [];
         this._addPlaceholders();
@@ -67,10 +69,10 @@ Ephy.Overview = class Overview
 
     _onKeyPress(event)
     {
-        if (event.which != 127)
+        if (event.which !== 127)
             return;
 
-        let item = document.activeElement;
+        const item = document.activeElement;
         if (item.classList.contains('overview-item')) {
             this._removeItem(item);
             event.preventDefault();
@@ -78,16 +80,16 @@ Ephy.Overview = class Overview
     }
 
     _addPlaceholders() {
-        let parentNode = document.getElementById('most-visited-grid');
-        let anchors = document.getElementsByTagName('a');
+        const parentNode = document.getElementById('most-visited-grid');
+        const anchors = document.getElementsByTagName('a');
 
         for (let i = anchors.length; i < 9; i++) {
-            let anchor = document.createElement('a');
+            const anchor = document.createElement('a');
             anchor.className = 'overview-item';
-            let span_thumbnail = document.createElement('span');
+            const span_thumbnail = document.createElement('span');
             span_thumbnail.className = 'overview-thumbnail';
             anchor.appendChild(span_thumbnail);
-            let span_title = document.createElement('span');
+            const span_title = document.createElement('span');
             span_title.className = 'overview-title';
             anchor.appendChild(span_title);
 
@@ -96,10 +98,10 @@ Ephy.Overview = class Overview
       }
 
     _removePlaceholders() {
-        let anchors = document.getElementsByTagName('a');
+        const anchors = document.getElementsByTagName('a');
 
-        for (let anchor of anchors) {
-            if (anchor.href == '')
+        for (const anchor of anchors) {
+            if (anchor.href === '')
                 document.removeChild(anchor);
         }
     }
@@ -111,7 +113,7 @@ Ephy.Overview = class Overview
         setTimeout(() => {
             item.parentNode.removeChild(item);
             for (let i = 0; i < this._items.length; i++) {
-                if (this._items[i].url() == item.href) {
+                if (this._items[i].url() === item.href) {
                     this._items.splice(i, 1);
                     break;
                 }
@@ -123,8 +125,8 @@ Ephy.Overview = class Overview
 
     _onURLsChanged(urls)
     {
-        let overview = document.getElementById('overview');
-        let grid = document.getElementById('most-visited-grid');
+        const overview = document.getElementById('overview');
+        const grid = document.getElementById('most-visited-grid');
         if (overview.classList.contains('overview-empty')) {
             while (grid.lastChild)
                 grid.removeChild(grid.lastChild);
@@ -133,17 +135,17 @@ Ephy.Overview = class Overview
 
         this._removePlaceholders();
         for (let i = 0; i < urls.length; i++) {
-            let url = urls[i];
+            const url = urls[i];
 
             let item;
             if (this._items[i]) {
                 item = this._items[i];
             } else {
                 Ephy.log('create an item for the url ' + url.url);
-                let anchor = document.createElement('a');
+                const anchor = document.createElement('a');
                 anchor.classList.add('overview-item');
-                let closeButton = document.createElement('div');
-                closeButton.title = Ephy._("Remove from overview");
+                const closeButton = document.createElement('div');
+                closeButton.title = Ephy._('Remove from overview');
                 closeButton.onclick = (event) => {
                     this._removeItem(anchor);
                     event.preventDefault();
@@ -151,10 +153,10 @@ Ephy.Overview = class Overview
                 closeButton.innerHTML = '';
                 closeButton.classList.add('overview-close-button');
                 anchor.appendChild(closeButton);
-                let thumbnailSpan = document.createElement('span');
+                const thumbnailSpan = document.createElement('span');
                 thumbnailSpan.classList.add('overview-thumbnail');
                 anchor.appendChild(thumbnailSpan);
-                let titleSpan = document.createElement('span');
+                const titleSpan = document.createElement('span');
                 titleSpan.classList.add('overview-title');
                 anchor.appendChild(titleSpan);
                 document.getElementById('most-visited-grid').appendChild(anchor);
@@ -168,21 +170,21 @@ Ephy.Overview = class Overview
         }
 
         while (this._items.length > urls.length) {
-            let item = this._items.pop();
+            const item = this._items.pop();
             item.detachFromParent();
         }
     }
 
     _onThumbnailChanged(url, path)
     {
-        if (this._items.length == 0) {
+        if (this._items.length === 0) {
             this._pendingThumbnailChanges.push({ url: url, path: path });
             return;
         }
 
         for (let i = 0; i < this._items.length; i++) {
-            let item = this._items[i];
-            if (item.url() == url) {
+            const item = this._items[i];
+            if (item.url() === url) {
                 item.setThumbnailPath(path);
                 return;
             }
@@ -191,14 +193,14 @@ Ephy.Overview = class Overview
 
     _onTitleChanged(url, title)
     {
-        if (this._items.length == 0) {
+        if (this._items.length === 0) {
             this._pendingTitleChanges.push({ url: url, title: title });
             return;
         }
 
         for (let i = 0; i < this._items.length; i++) {
-            let item = this._items[i];
-            if (item.url() == url) {
+            const item = this._items[i];
+            if (item.url() === url) {
                 item.setTitle(title);
                 return;
             }
@@ -215,7 +217,7 @@ Ephy.Overview.Item = class OverviewItem
         this._thumbnail = null;
 
         for (let i = 0; i < this._item.childNodes.length; i++) {
-            let child = this._item.childNodes[i];
+            const child = this._item.childNodes[i];
             if (!(child instanceof Element))
                 continue;
 
@@ -251,11 +253,11 @@ Ephy.Overview.Item = class OverviewItem
 
     thumbnailPath()
     {
-        let style = this._thumbnail.style;
+        const style = this._thumbnail.style;
         if (style.isPropertyImplicit('background'))
             return null;
 
-        let background = style.getPropertyValue('background');
+        const background = style.getPropertyValue('background');
         if (!background)
             return null;
 
