@@ -26,12 +26,11 @@ G_BEGIN_DECLS
 
 #define EPHY_TYPE_NOTIFICATION_CONTAINER (ephy_notification_container_get_type ())
 
-G_DECLARE_FINAL_TYPE (EphyNotificationContainer, ephy_notification_container, EPHY, NOTIFICATION_CONTAINER, GtkRevealer)
+G_DECLARE_FINAL_TYPE (EphyNotificationContainer, ephy_notification_container, EPHY, NOTIFICATION_CONTAINER, GtkBin)
 
 EphyNotificationContainer *ephy_notification_container_get_default      (void);
 
 void                       ephy_notification_container_add_notification (EphyNotificationContainer *self,
                                                                          GtkWidget                 *notification);
-guint                      ephy_notification_container_get_num_children (EphyNotificationContainer *self);
 
 G_END_DECLS
