@@ -23,14 +23,6 @@
 
 G_BEGIN_DECLS
 
-typedef enum
-{
-  EPHY_PREFS_UI_TABS_BAR_POSITION_TOP,
-  EPHY_PREFS_UI_TABS_BAR_POSITION_BOTTOM,
-  EPHY_PREFS_UI_TABS_BAR_POSITION_LEFT,
-  EPHY_PREFS_UI_TABS_BAR_POSITION_RIGHT
-} EphyPrefsUITabsBarPosition;
-
 typedef enum
 {
   EPHY_PREFS_READER_FONT_STYLE_SANS,
@@ -46,7 +38,6 @@ typedef enum
 typedef enum
 {
   EPHY_PREFS_RESTORE_SESSION_POLICY_ALWAYS,
-  EPHY_PREFS_RESTORE_SESSION_POLICY_NEVER,
   EPHY_PREFS_RESTORE_SESSION_POLICY_CRASHED
 } EphyPrefsRestoreSessionPolicy;
 
@@ -66,7 +57,6 @@ typedef enum
 
 #define EPHY_PREFS_UI_SCHEMA                     "org.gnome.Epiphany.ui"
 #define EPHY_PREFS_UI_EXPAND_TABS_BAR            "expand-tabs-bar"
-#define EPHY_PREFS_UI_TABS_BAR_POSITION          "tabs-bar-position"
 #define EPHY_PREFS_UI_TABS_BAR_VISIBILITY_POLICY "tabs-bar-visibility-policy"
 #define EPHY_PREFS_UI_KEEP_WINDOW_OPEN           "keep-window-open"
 
@@ -100,14 +90,12 @@ static const char * const ephy_prefs_state_schema[] = {
 #define EPHY_PREFS_WEB_ENABLE_USER_JS               "enable-user-js"
 #define EPHY_PREFS_WEB_ENABLE_POPUPS                "enable-popups"
 #define EPHY_PREFS_WEB_ENABLE_SPELL_CHECKING        "enable-spell-checking"
-#define EPHY_PREFS_WEB_ENABLE_SMOOTH_SCROLLING      "enable-smooth-scrolling"
 #define EPHY_PREFS_WEB_USER_AGENT                   "user-agent"
 #define EPHY_PREFS_WEB_DEFAULT_ENCODING             "default-encoding"
 #define EPHY_PREFS_WEB_ENABLE_ADBLOCK               "enable-adblock"
 #define EPHY_PREFS_WEB_REMEMBER_PASSWORDS           "remember-passwords"
 #define EPHY_PREFS_WEB_ENABLE_SITE_SPECIFIC_QUIRKS  "enable-site-specific-quirks"
 #define EPHY_PREFS_WEB_ENABLE_SAFE_BROWSING         "enable-safe-browsing"
-#define EPHY_PREFS_WEB_GSB_API_KEY                  "gsb-api-key"
 #define EPHY_PREFS_WEB_ENABLE_ITP                   "enable-itp"
 #define EPHY_PREFS_WEB_ENABLE_WEBSITE_DATA_STORAGE  "enable-website-data-storage"
 #define EPHY_PREFS_WEB_DEFAULT_ZOOM_LEVEL           "default-zoom-level"
@@ -118,6 +106,8 @@ static const char * const ephy_prefs_state_schema[] = {
 #define EPHY_PREFS_WEB_HARDWARE_ACCELERATION_POLICY "hardware-acceleration-policy"
 #define EPHY_PREFS_WEB_ASK_ON_DOWNLOAD              "ask-on-download"
 #define EPHY_PREFS_WEB_SWITCH_TO_NEW_TAB            "switch-to-new-tab"
+#define EPHY_PREFS_WEB_ENABLE_WEBEXTENSIONS         "enable-webextensions"
+#define EPHY_PREFS_WEB_WEBEXTENSIONS_ACTIVE         "webextensions-active"
 
 static const char * const ephy_prefs_web_schema[] = {
   EPHY_PREFS_WEB_FONT_MIN_SIZE,
@@ -130,14 +120,12 @@ static const char * const ephy_prefs_web_schema[] = {
   EPHY_PREFS_WEB_ENABLE_USER_JS,
   EPHY_PREFS_WEB_ENABLE_POPUPS,
   EPHY_PREFS_WEB_ENABLE_SPELL_CHECKING,
-  EPHY_PREFS_WEB_ENABLE_SMOOTH_SCROLLING,
   EPHY_PREFS_WEB_USER_AGENT,
   EPHY_PREFS_WEB_DEFAULT_ENCODING,
   EPHY_PREFS_WEB_ENABLE_ADBLOCK,
   EPHY_PREFS_WEB_REMEMBER_PASSWORDS,
   EPHY_PREFS_WEB_ENABLE_SITE_SPECIFIC_QUIRKS,
   EPHY_PREFS_WEB_ENABLE_SAFE_BROWSING,
-  EPHY_PREFS_WEB_GSB_API_KEY,
   EPHY_PREFS_WEB_ENABLE_ITP,
   EPHY_PREFS_WEB_ENABLE_WEBSITE_DATA_STORAGE,
   EPHY_PREFS_WEB_DEFAULT_ZOOM_LEVEL,
@@ -148,6 +136,7 @@ static const char * const ephy_prefs_web_schema[] = {
   EPHY_PREFS_WEB_HARDWARE_ACCELERATION_POLICY,
   EPHY_PREFS_WEB_ASK_ON_DOWNLOAD,
   EPHY_PREFS_WEB_SWITCH_TO_NEW_TAB,
+  EPHY_PREFS_WEB_ENABLE_WEBEXTENSIONS,
 };
 
 #define EPHY_PREFS_SCHEMA                             "org.gnome.Epiphany"
@@ -158,10 +147,12 @@ static const char * const ephy_prefs_web_schema[] = {
 #define EPHY_PREFS_RESTORE_SESSION_POLICY             "restore-session-policy"
 #define EPHY_PREFS_RESTORE_SESSION_DELAYING_LOADS     "restore-session-delaying-loads"
 #define EPHY_PREFS_CONTENT_FILTERS                    "content-filters"
-#define EPHY_PREFS_SEARCH_ENGINES                     "search-engines"
+#define EPHY_PREFS_SEARCH_ENGINES                     "search-engine-providers"
 #define EPHY_PREFS_DEFAULT_SEARCH_ENGINE              "default-search-engine"
 #define EPHY_PREFS_ASK_FOR_DEFAULT                    "ask-for-default"
 #define EPHY_PREFS_START_IN_INCOGNITO_MODE            "start-in-incognito-mode"
+#define EPHY_PREFS_ACTIVE_CLEAR_DATA_ITEMS            "active-clear-data-items"
+#define EPHY_PREFS_USE_GOOGLE_SEARCH_SUGGESTIONS      "use-google-search-suggestions"
 
 #define EPHY_PREFS_LOCKDOWN_SCHEMA            "org.gnome.Epiphany.lockdown"
 #define EPHY_PREFS_LOCKDOWN_FULLSCREEN        "disable-fullscreen"
@@ -194,10 +185,11 @@ static const char * const ephy_prefs_web_schema[] = {
 #define EPHY_PREFS_SYNC_TOKEN_SERVER      "sync-token-server"
 #define EPHY_PREFS_SYNC_ACCOUNTS_SERVER   "sync-accounts-server"
 
-#define EPHY_PREFS_WEB_APP_SCHEMA            "org.gnome.Epiphany.webapp"
-#define EPHY_PREFS_WEB_APP_ADDITIONAL_URLS   "additional-urls"
-#define EPHY_PREFS_WEB_APP_MOBILE_CAPABLE    "mobile-capable"
-#define EPHY_PREFS_WEB_APP_RUN_IN_BACKGROUND "run-in-background"
+#define EPHY_PREFS_WEB_APP_SCHEMA                  "org.gnome.Epiphany.webapp"
+#define EPHY_PREFS_WEB_APP_ADDITIONAL_URLS         "additional-urls"
+#define EPHY_PREFS_WEB_APP_SHOW_NAVIGATION_BUTTONS "show-navigation-buttons"
+#define EPHY_PREFS_WEB_APP_RUN_IN_BACKGROUND       "run-in-background"
+#define EPHY_PREFS_WEB_APP_SYSTEM                  "system"
 
 static struct {
   const char *schema;
