@@ -21,15 +21,17 @@
 #include "config.h"
 #include "ephy-user-agent.h"
 
+#include "ephy-file-helpers.h"
 #include "ephy-settings.h"
 
 #include <webkit2/webkit2.h>
 
 const char *
-ephy_user_agent_get_internal (void)
+ephy_user_agent_get (void)
 {
-  WebKitSettings *settings;
   static char *user_agent = NULL;
+  WebKitSettings *settings;
+  gboolean web_app;
 
   if (user_agent)
     return user_agent;
@@ -41,8 +43,12 @@ ephy_user_agent_get_internal (void)
     g_free (user_agent);
   }
 
+  web_app = ephy_profile_dir_is_web_application ();
+
   settings = webkit_settings_new ();
-  user_agent = g_strdup_printf ("%s Epiphany/605.1.15", webkit_settings_get_user_agent (settings));
+  user_agent = g_strdup_printf ("%s%s",
+                                webkit_settings_get_user_agent (settings),
+                                web_app ? " (Web App)" : "");
   g_object_unref (settings);
 
   return user_agent;
