@@ -29,97 +29,166 @@
 #include <errno.h>
 #include <gio/gio.h>
 #include <glib/gstdio.h>
-#include <libsoup/soup.h>
 #include <stdlib.h>
 #include <string.h>
 #include <fcntl.h>
 
-/* Web Apps are installed in the default config dir of the user.
- * Every app has its own profile directory. To create a web app
- * an id needs to be generated using the given name as
- * <normalized-name>-checksum.
+/* Web apps are installed in the default data dir of the user. Every
+ * app has its own profile directory. To create a web app, an ID needs
+ * to be generated using the given name as <normalized-name>-checksum.
+ * The ID is used to uniquely identify the app.
  *
- * The id is used to uniquely identify the app.
- *  - Program name: org.gnome.Epiphany.WebApp-<id>
- *  - Profile directory: <program-name>
- *  - Desktop file: <profile-dir>/<program-name>.desktop
+ *  - Name: the user-visible pretty app name
+ *  - Normalized name: lowercase name
+ *  - Checksum: SHA-1 of name
+ *  - ID: <normalized-name>-<checksum>
+ *  - GApplication ID: see below
+ *  - Profile directory: <gapplication-id>
+ *  - Desktop file: <profile-dir>/<gapplication-id>.desktop
+ *
+ * Note that our ID and GApplication ID are different. Yes, this is confusing.
+ *
+ * For GApplication ID, there are two cases:
+ *
+ *  - If <id> is safe to use in a D-Bus identifier,
+ *    then: GApplication ID is org.gnome.Epiphany.WebApp-<id>
+ *  - otherwise: GAppliation ID is org.gnome.Epiphany.WebApp-<checksum>
  *
  * System web applications have a profile dir without a desktop file.
  */
 
-#define EPHY_WEB_APP_PROGRAM_NAME_PREFIX "org.gnome.Epiphany.WebApp-"
+#define EPHY_WEB_APP_GAPPLICATION_ID_PREFIX "org.gnome.Epiphany.WebApp-"
 
 char *
 ephy_web_application_get_app_id_from_name (const char *name)
 {
-  char *normal_id;
-  char *checksum;
-  char *id;
+  g_autofree char *normal_id = NULL;
+  g_autofree char *checksum = NULL;
 
+  /* FIXME: We should not use hyphens in app IDs. Sadly, changing this requires
+   * a new migration.
+   */
   normal_id = g_utf8_strdown (name, -1);
   g_strdelimit (normal_id, " ", '-');
   g_strdelimit (normal_id, G_DIR_SEPARATOR_S, '-');
 
   checksum = g_compute_checksum_for_string (G_CHECKSUM_SHA1, name, -1);
-  id = g_strdup_printf ("%s-%s", normal_id, checksum);
-
-  g_free (normal_id);
-  g_free (checksum);
-
-  return id;
+  return g_strdup_printf ("%s-%s", normal_id, checksum);
 }
 
 static char *
 get_encoded_path (const char *path)
 {
-  char *encoded;
-  GError *error = NULL;
+  g_autofree char *encoded = NULL;
+  g_autoptr (GError) error = NULL;
 
   encoded = g_filename_from_utf8 (path, -1, NULL, NULL, &error);
   if (error) {
     g_warning ("%s", error->message);
-    g_error_free (error);
     return NULL;
   }
 
-  return encoded;
+  return g_steal_pointer (&encoded);
+}
+
+static const char *
+get_app_id_from_gapplication_id (const char *name)
+{
+  if (!g_str_has_prefix (name, EPHY_WEB_APP_GAPPLICATION_ID_PREFIX)) {
+    g_warning ("GApplication ID %s does not begin with required prefix %s", name, EPHY_WEB_APP_GAPPLICATION_ID_PREFIX);
+    return NULL;
+  }
+
+  return name + strlen (EPHY_WEB_APP_GAPPLICATION_ID_PREFIX);
+}
+
+static char *
+get_gapplication_id_from_id (const char *id)
+{
+  g_autofree char *gapplication_id = NULL;
+  const char *final_hyphen;
+  const char *checksum;
+
+  /* FIXME: Ideally we would convert hyphens to underscores here, because
+   * hyphens are not very friendly to D-Bus. However, changing this
+   * would require a new profile migration, because the GApplication ID
+   * must exactly match the desktop file basename.
+   *
+   * If we're willing to do another migration in the future, then we
+   * should drop this path and always return just the prefix plus hash,
+   * using underscores rather than hyphens.
+   */
+  gapplication_id = g_strconcat (EPHY_WEB_APP_GAPPLICATION_ID_PREFIX, id, NULL);
+  if (g_application_id_is_valid (gapplication_id))
+    return g_steal_pointer (&gapplication_id);
+
+  /* Split ID into: <normalized-name>-<checksum> */
+  final_hyphen = strrchr (id, '-');
+  if (!final_hyphen) {
+    g_warning ("Web app ID %s is broken: must contain a hyphen", id);
+    return NULL;
+  }
+  checksum = final_hyphen + 1;
+
+  if (*checksum == '\0') {
+    g_warning ("Web app ID %s is broken: should end with checksum, not hyphen", id);
+    return NULL;
+  }
+
+  /* We'll simply omit the <normalized-name> from the app ID, in order
+   * to avoid problematic characters. Ideally we would use an underscore
+   * here too, rather than a hyphen, but let's be consistent with
+   * existing web apps.
+   */
+  g_clear_pointer (&gapplication_id, g_free);
+  gapplication_id = g_strconcat (EPHY_WEB_APP_GAPPLICATION_ID_PREFIX, checksum, NULL);
+
+  if (!g_application_id_is_valid (gapplication_id)) {
+    g_warning ("Web app ID %s is broken: derived GApplication ID %s is not a valid app ID (is the final component alphanumeric?)", id, gapplication_id);
+    return NULL;
+  }
+
+  return g_steal_pointer (&gapplication_id);
 }
 
 static char *
 get_app_profile_directory_name (const char *id)
 {
-  char *profile_dir;
-  char *encoded;
+  g_autofree char *gapplication_id = NULL;
 
-  profile_dir = g_strconcat (EPHY_WEB_APP_PROGRAM_NAME_PREFIX, id, NULL);
-  encoded = get_encoded_path (profile_dir);
-  g_free (profile_dir);
+  gapplication_id = get_gapplication_id_from_id (id);
+  if (!gapplication_id)
+    g_error ("Failed to get GApplication ID from app ID %s", id);
 
-  return encoded;
+  return get_encoded_path (gapplication_id);
 }
 
 static char *
 get_app_desktop_filename (const char *id)
 {
-  char *filename;
-  char *encoded;
+  g_autofree char *gapplication_id = NULL;
+  g_autofree char *filename = NULL;
 
-  filename = g_strconcat (EPHY_WEB_APP_PROGRAM_NAME_PREFIX, id, ".desktop", NULL);
-  encoded = get_encoded_path (filename);
-  g_free (filename);
+  /* Warning: the GApplication ID must exactly match the desktop file's
+   * basename. Don't overthink this or stuff will break, e.g. GNotification.
+   */
+  gapplication_id = get_gapplication_id_from_id (id);
+  if (!gapplication_id)
+    g_error ("Failed to get GApplication ID from app ID %s", id);
 
-  return encoded;
+  filename = g_strconcat (gapplication_id, ".desktop", NULL);
+  return get_encoded_path (filename);
 }
 
 const char *
-ephy_web_application_get_program_name_from_profile_directory (const char *profile_dir)
+ephy_web_application_get_gapplication_id_from_profile_directory (const char *profile_dir)
 {
   const char *name;
 
   /* Just get the basename */
   name = strrchr (profile_dir, G_DIR_SEPARATOR);
   if (name == NULL) {
-    g_warning ("Profile directoroy %s is not a valid path", profile_dir);
+    g_warning ("Profile directory %s is not a valid path", profile_dir);
     return NULL;
   }
 
@@ -129,32 +198,21 @@ ephy_web_application_get_program_name_from_profile_directory (const char *profil
   if (g_str_has_prefix (name, "app-"))
     name += strlen ("app-");
 
-  if (!g_str_has_prefix (name, EPHY_WEB_APP_PROGRAM_NAME_PREFIX)) {
-    g_warning ("Profile directory %s does not begin with required web app prefix %s", profile_dir, EPHY_WEB_APP_PROGRAM_NAME_PREFIX);
+  if (!g_str_has_prefix (name, EPHY_WEB_APP_GAPPLICATION_ID_PREFIX)) {
+    g_warning ("Profile directory %s does not begin with required web app prefix %s", profile_dir, EPHY_WEB_APP_GAPPLICATION_ID_PREFIX);
     return NULL;
   }
 
   return name;
 }
 
-static const char *
-get_app_id_from_program_name (const char *name)
-{
-  if (!g_str_has_prefix (name, EPHY_WEB_APP_PROGRAM_NAME_PREFIX)) {
-    g_warning ("Program name %s does not begin with required prefix %s", name, EPHY_WEB_APP_PROGRAM_NAME_PREFIX);
-    return NULL;
-  }
-
-  return name + strlen (EPHY_WEB_APP_PROGRAM_NAME_PREFIX);
-}
-
 static const char *
 get_app_id_from_profile_directory (const char *profile_dir)
 {
-  const char *program_name;
+  const char *gapplication_id;
 
-  program_name = ephy_web_application_get_program_name_from_profile_directory (profile_dir);
-  return program_name ? get_app_id_from_program_name (program_name) : NULL;
+  gapplication_id = ephy_web_application_get_gapplication_id_from_profile_directory (profile_dir);
+  return gapplication_id ? get_app_id_from_gapplication_id (gapplication_id) : NULL;
 }
 
 static char *
@@ -279,14 +337,16 @@ create_desktop_file (const char *id,
                      const char *profile_dir,
                      GdkPixbuf  *icon)
 {
-  GKeyFile *file = NULL;
-  char *exec_string;
-  char *data = NULL;
-  char *filename, *apps_path, *desktop_file_path = NULL;
-  char *link_path;
-  char *wm_class;
-  GFile *link;
-  GError *error = NULL;
+  g_autofree char *filename = NULL;
+  g_autoptr (GKeyFile) file = NULL;
+  g_autofree char *exec_string = NULL;
+  g_autofree char *wm_class = NULL;
+  g_autofree char *data = NULL;
+  g_autofree char *desktop_file_path = NULL;
+  g_autofree char *apps_path = NULL;
+  g_autofree char *link_path = NULL;
+  g_autoptr (GFile) link = NULL;
+  g_autoptr (GError) error = NULL;
 
   g_assert (profile_dir);
 
@@ -296,20 +356,19 @@ create_desktop_file (const char *id,
 
   file = g_key_file_new ();
   g_key_file_set_value (file, "Desktop Entry", "Name", name);
-  exec_string = g_strdup_printf ("epiphany --application-mode --profile=\"%s\" %s",
+  exec_string = g_strdup_printf ("epiphany --application-mode \"--profile=%s\" %s",
                                  profile_dir,
                                  address);
   g_key_file_set_value (file, "Desktop Entry", "Exec", exec_string);
-  g_free (exec_string);
   g_key_file_set_value (file, "Desktop Entry", "StartupNotify", "true");
   g_key_file_set_value (file, "Desktop Entry", "Terminal", "false");
   g_key_file_set_value (file, "Desktop Entry", "Type", "Application");
   g_key_file_set_value (file, "Desktop Entry", "Categories", "GNOME;GTK;");
 
   if (icon) {
-    GOutputStream *stream;
-    char *path;
-    GFile *image;
+    g_autoptr (GOutputStream) stream = NULL;
+    g_autofree char *path = NULL;
+    g_autoptr (GFile) image = NULL;
 
     path = g_build_filename (profile_dir, EPHY_WEB_APP_ICON_NAME, NULL);
     image = g_file_new_for_path (path);
@@ -317,23 +376,19 @@ create_desktop_file (const char *id,
     stream = (GOutputStream *)g_file_create (image, 0, NULL, NULL);
     gdk_pixbuf_save_to_stream (icon, stream, "png", NULL, NULL, NULL);
     g_key_file_set_value (file, "Desktop Entry", "Icon", path);
-
-    g_object_unref (stream);
-    g_object_unref (image);
-    g_free (path);
   }
 
-  wm_class = g_strconcat (EPHY_WEB_APP_PROGRAM_NAME_PREFIX, id, NULL);
+  wm_class = g_strconcat (EPHY_WEB_APP_GAPPLICATION_ID_PREFIX, id, NULL);
   g_key_file_set_value (file, "Desktop Entry", "StartupWMClass", wm_class);
-  g_free (wm_class);
+
+  g_key_file_set_value (file, "Desktop Entry", "X-Purism-FormFactor", "Workstation;Mobile;");
+
   data = g_key_file_to_data (file, NULL, NULL);
 
   desktop_file_path = g_build_filename (profile_dir, filename, NULL);
 
-  if (!g_file_set_contents (desktop_file_path, data, -1, NULL)) {
-    g_free (desktop_file_path);
-    desktop_file_path = NULL;
-  }
+  if (!g_file_set_contents (desktop_file_path, data, -1, NULL))
+    g_clear_pointer (&desktop_file_path, g_free);
 
   /* Create a symlink in XDG_DATA_DIR/applications for the Shell to
    * pick up this application. */
@@ -341,20 +396,12 @@ create_desktop_file (const char *id,
   if (ephy_ensure_dir_exists (apps_path, &error)) {
     link_path = g_build_filename (apps_path, filename, NULL);
     link = g_file_new_for_path (link_path);
-    g_free (link_path);
     g_file_make_symbolic_link (link, desktop_file_path, NULL, NULL);
-    g_object_unref (link);
   } else {
     g_warning ("Error creating application symlink: %s", error->message);
-    g_error_free (error);
   }
 
-  g_free (apps_path);
-  g_free (filename);
-  g_free (data);
-  g_key_file_free (file);
-
-  return desktop_file_path;
+  return g_steal_pointer (&desktop_file_path);
 }
 
 /**
@@ -467,7 +514,7 @@ ephy_web_application_ensure_for_app_info (GAppInfo *app_info)
     if (error)
       g_warning ("Couldn't copy desktop file: %s", error->message);
 
-    ephy_web_application_initialize_settings (profile_dir, EPHY_WEB_APPLICATION_NONE);
+    ephy_web_application_initialize_settings (profile_dir, EPHY_WEB_APPLICATION_SYSTEM);
   }
 
   return g_steal_pointer (&profile_dir);
@@ -476,84 +523,44 @@ ephy_web_application_ensure_for_app_info (GAppInfo *app_info)
 void
 ephy_web_application_setup_from_profile_directory (const char *profile_directory)
 {
-  const char *program_name;
+  const char *gapplication_id;
   const char *id;
-  char *app_icon;
-  char *desktop_basename;
-  char *desktop_filename;
-  GDesktopAppInfo *desktop_info;
+  g_autofree char *desktop_basename = NULL;
+  g_autofree char *desktop_filename = NULL;
+  g_autoptr (GDesktopAppInfo) desktop_info = NULL;
 
   g_assert (profile_directory != NULL);
 
-  program_name = ephy_web_application_get_program_name_from_profile_directory (profile_directory);
-  if (!program_name)
-    exit (1);
+  gapplication_id = ephy_web_application_get_gapplication_id_from_profile_directory (profile_directory);
+  if (!gapplication_id)
+    g_error ("Failed to get GApplication ID from profile directory %s", profile_directory);
 
-  g_set_prgname (program_name);
+  /* FIXME: is this actually necessary? Probably not? */
+  g_set_prgname (gapplication_id);
 
-  id = get_app_id_from_program_name (program_name);
+  id = get_app_id_from_gapplication_id (gapplication_id);
   if (!id)
-    exit (1);
+    g_error ("Failed to get app ID from GApplication ID %s", gapplication_id);
 
   /* Get display name from desktop file */
   desktop_basename = get_app_desktop_filename (id);
   desktop_filename = g_build_filename (profile_directory, desktop_basename, NULL);
   desktop_info = g_desktop_app_info_new_from_filename (desktop_filename);
-  if (!desktop_info) {
-    g_warning ("Required desktop file not present at %s", desktop_filename);
-    exit (1);
-  }
+  if (!desktop_info)
+    g_error ("Required desktop file not present at %s", desktop_filename);
   g_set_application_name (g_app_info_get_name (G_APP_INFO (desktop_info)));
-
-  app_icon = g_build_filename (profile_directory, EPHY_WEB_APP_ICON_NAME, NULL);
-  gtk_window_set_default_icon_from_file (app_icon, NULL);
-
-  /* We need to re-set this because we have already parsed the
-   * options, which inits GTK+ and sets this as a side effect.
-   */
-  gdk_set_program_class (program_name);
-
-  g_free (app_icon);
-  g_free (desktop_basename);
-  g_free (desktop_filename);
-  g_object_unref (desktop_info);
 }
 
 void
 ephy_web_application_setup_from_desktop_file (GDesktopAppInfo *desktop_info)
 {
   GAppInfo *app_info;
-  const char *wm_class;
-  GIcon *icon;
 
   g_assert (G_IS_DESKTOP_APP_INFO (desktop_info));
 
   app_info = G_APP_INFO (desktop_info);
   g_set_prgname (g_app_info_get_name (app_info));
   g_set_application_name (g_app_info_get_display_name (app_info));
-
-  icon = g_app_info_get_icon (app_info);
-  if (G_IS_FILE_ICON (icon)) {
-    GFile *file = g_file_icon_get_file (G_FILE_ICON (icon));
-    char *path = file ? g_file_get_path (file) : NULL;
-
-    if (path) {
-      gtk_window_set_default_icon_from_file (path, NULL);
-      g_free (path);
-    }
-    g_clear_object (&file);
-  } else if (G_IS_THEMED_ICON (icon)) {
-    const char * const *names = g_themed_icon_get_names (G_THEMED_ICON (icon));
-    if (names)
-      gtk_window_set_default_icon_name (names[0]);
-  }
-
-  /* We need to re-set this because we have already parsed the
-   * options, which inits GTK+ and sets this as a side effect.
-   */
-  wm_class = g_desktop_app_info_get_startup_wm_class (desktop_info);
-  if (wm_class)
-    gdk_set_program_class (wm_class);
 }
 
 void
@@ -567,21 +574,20 @@ ephy_web_application_free (EphyWebApplication *app)
   g_free (app);
 }
 
-
 EphyWebApplication *
 ephy_web_application_for_profile_directory (const char *profile_dir)
 {
-  EphyWebApplication *app;
-  char *desktop_file_path;
+  g_autoptr (EphyWebApplication) app = NULL;
+  g_autofree char *desktop_file_path = NULL;
   const char *id;
-  GDesktopAppInfo *desktop_info;
+  g_autoptr (GDesktopAppInfo) desktop_info = NULL;
   const char *exec;
   int argc;
-  char **argv;
-  GFile *file;
-  GFileInfo *file_info;
+  g_auto (GStrv) argv = NULL;
+  g_autoptr (GFile) file = NULL;
+  g_autoptr (GFileInfo) file_info = NULL;
   guint64 created;
-  GDate *date;
+  g_autoptr (GDate) date = NULL;
 
   id = get_app_id_from_profile_directory (profile_dir);
   if (!id)
@@ -594,20 +600,15 @@ ephy_web_application_for_profile_directory (const char *profile_dir)
   desktop_file_path = g_build_filename (profile_dir, app->desktop_file, NULL);
   desktop_info = g_desktop_app_info_new_from_filename (desktop_file_path);
   if (!desktop_info) {
-    ephy_web_application_free (app);
-    g_free (desktop_file_path);
+    g_clear_pointer (&app, ephy_web_application_free); /* avoid a scan-build warning */
     return NULL;
   }
 
   app->name = g_strdup (g_app_info_get_name (G_APP_INFO (desktop_info)));
   app->icon_url = g_desktop_app_info_get_string (desktop_info, "Icon");
   exec = g_app_info_get_commandline (G_APP_INFO (desktop_info));
-  if (g_shell_parse_argv (exec, &argc, &argv, NULL)) {
+  if (g_shell_parse_argv (exec, &argc, &argv, NULL))
     app->url = g_strdup (argv[argc - 1]);
-    g_strfreev (argv);
-  }
-
-  g_object_unref (desktop_info);
 
   file = g_file_new_for_path (desktop_file_path);
 
@@ -619,19 +620,13 @@ ephy_web_application_for_profile_directory (const char *profile_dir)
   g_date_set_time_t (date, (time_t)created);
   g_date_strftime (app->install_date, 127, "%x", date);
 
-  g_date_free (date);
-  g_object_unref (file);
-  g_object_unref (file_info);
-  g_free (desktop_file_path);
-
-  return app;
+  return g_steal_pointer (&app);
 }
 
 static GList *
 ephy_web_application_get_application_list_internal (gboolean only_legacy)
 {
-  GFileEnumerator *children = NULL;
-  GFileInfo *info;
+  g_autoptr (GFileEnumerator) children = NULL;
   GList *applications = NULL;
   g_autofree char *parent_directory_path = NULL;
   g_autoptr (GFile) parent_directory = NULL;
@@ -648,15 +643,18 @@ ephy_web_application_get_application_list_internal (gboolean only_legacy)
   if (!children)
     return NULL;
 
-  info = g_file_enumerator_next_file (children, NULL, NULL);
-  while (info) {
+  for (;;) {
+    g_autoptr (GFileInfo) info = g_file_enumerator_next_file (children, NULL, NULL);
     const char *name;
 
+    if (!info)
+      break;
+
     name = g_file_info_get_name (info);
     if ((only_legacy && g_str_has_prefix (name, "app-")) ||
-        (!only_legacy && g_str_has_prefix (name, EPHY_WEB_APP_PROGRAM_NAME_PREFIX))) {
-      EphyWebApplication *app;
-      char *profile_dir;
+        (!only_legacy && g_str_has_prefix (name, EPHY_WEB_APP_GAPPLICATION_ID_PREFIX))) {
+      g_autoptr (EphyWebApplication) app = NULL;
+      g_autofree char *profile_dir = NULL;
 
       profile_dir = g_build_filename (parent_directory_path, name, NULL);
       app = ephy_web_application_for_profile_directory (profile_dir);
@@ -664,23 +662,13 @@ ephy_web_application_get_application_list_internal (gboolean only_legacy)
         if (!only_legacy) {
           g_autofree char *app_file = g_build_filename (profile_dir, ".app", NULL);
           if (g_file_test (app_file, G_FILE_TEST_EXISTS))
-            applications = g_list_prepend (applications, app);
-          else
-            g_free (app);
+            applications = g_list_prepend (applications, g_steal_pointer (&app));
         } else
-          applications = g_list_prepend (applications, app);
+          applications = g_list_prepend (applications, g_steal_pointer (&app));
       }
-
-      g_free (profile_dir);
     }
-
-    g_object_unref (info);
-
-    info = g_file_enumerator_next_file (children, NULL, NULL);
   }
 
-  g_object_unref (children);
-
   return g_list_reverse (applications);
 }
 
@@ -739,100 +727,93 @@ ephy_web_application_free_application_list (GList *list)
 gboolean
 ephy_web_application_exists (const char *id)
 {
-  char *profile_dir;
-  gboolean profile_exists;
+  g_autofree char *profile_dir = NULL;
 
   profile_dir = ephy_web_application_get_profile_directory (id);
-  profile_exists = g_file_test (profile_dir, G_FILE_TEST_IS_DIR);
-  g_free (profile_dir);
-
-  return profile_exists;
+  return g_file_test (profile_dir, G_FILE_TEST_IS_DIR);
 }
 
 void
 ephy_web_application_initialize_settings (const char                *profile_directory,
                                           EphyWebApplicationOptions  options)
 {
-  GSettings *settings;
-  GSettings *web_app_settings;
-  char *name;
-  char *path;
+  g_autoptr (GSettings) settings = NULL;
+  g_autoptr (GSettings) web_app_settings = NULL;
+  g_autofree char *name = NULL;
+  g_autofree char *path = NULL;
 
   name = g_path_get_basename (profile_directory);
   settings = g_settings_new_with_path (EPHY_PREFS_WEB_SCHEMA, "/org/gnome/epiphany/web/");
 
   path = g_build_path ("/", "/org/gnome/epiphany/web-apps/", name, "web/", NULL);
   web_app_settings = g_settings_new_with_path (EPHY_PREFS_WEB_SCHEMA, path);
-  g_free (path);
 
   for (guint i = 0; i < G_N_ELEMENTS (ephy_prefs_web_schema); i++) {
-    GVariant *value;
+    g_autoptr (GVariant) value = NULL;
 
     value = g_settings_get_value (settings, ephy_prefs_web_schema[i]);
     g_settings_set_value (web_app_settings, ephy_prefs_web_schema[i], value);
-    g_variant_unref (value);
   }
 
-  g_object_unref (settings);
-  g_object_unref (web_app_settings);
-
+  g_clear_object (&settings);
   settings = g_settings_new_with_path (EPHY_PREFS_STATE_SCHEMA, "/org/gnome/epiphany/state/");
 
+  g_clear_pointer (&path, g_free);
   path = g_build_path ("/", "/org/gnome/epiphany/web-apps/", name, "state/", NULL);
+  g_clear_object (&web_app_settings);
   web_app_settings = g_settings_new_with_path (EPHY_PREFS_STATE_SCHEMA, path);
-  g_free (path);
 
   for (guint i = 0; i < G_N_ELEMENTS (ephy_prefs_state_schema); i++) {
-    GVariant *value;
+    g_autoptr (GVariant) value = NULL;
 
     value = g_settings_get_value (settings, ephy_prefs_state_schema[i]);
     g_settings_set_value (web_app_settings, ephy_prefs_state_schema[i], value);
-    g_variant_unref (value);
   }
 
-  g_object_unref (settings);
-  g_object_unref (web_app_settings);
-
-  if (options & EPHY_WEB_APPLICATION_MOBILE_CAPABLE) {
+  if (options) {
+    g_clear_pointer (&path, g_free);
     path = g_build_path ("/", "/org/gnome/epiphany/web-apps/", name, "webapp/", NULL);
+    g_clear_object (&web_app_settings);
     web_app_settings = g_settings_new_with_path (EPHY_PREFS_WEB_APP_SCHEMA, path);
-    g_free (path);
 
-    g_settings_set_boolean (web_app_settings, EPHY_PREFS_WEB_APP_MOBILE_CAPABLE, TRUE);
-    g_object_unref (web_app_settings);
+    if (options & EPHY_WEB_APPLICATION_MOBILE_CAPABLE)
+      g_settings_set_boolean (web_app_settings, EPHY_PREFS_WEB_APP_SHOW_NAVIGATION_BUTTONS, TRUE);
+
+    if (options & EPHY_WEB_APPLICATION_SYSTEM)
+      g_settings_set_boolean (web_app_settings, EPHY_PREFS_WEB_APP_SYSTEM, TRUE);
   }
-
-  g_free (name);
 }
 
 static gboolean
 urls_have_same_origin (const char *a_url,
                        const char *b_url)
 {
-  SoupURI *a_uri, *b_uri;
-  gboolean retval = FALSE;
+  g_autoptr (GUri) a_uri = NULL;
+  g_autoptr (GUri) b_uri = NULL;
 
-  a_uri = soup_uri_new (a_url);
-  if (!a_uri)
-    return retval;
+  a_uri = g_uri_parse (a_url, G_URI_FLAGS_NONE, NULL);
+  if (!a_uri || !g_uri_get_host (a_uri))
+    return FALSE;
 
-  b_uri = soup_uri_new (b_url);
-  if (b_uri) {
-    retval = a_uri->host && b_uri->host && soup_uri_host_equal (a_uri, b_uri);
-    soup_uri_free (b_uri);
-  }
+  b_uri = g_uri_parse (b_url, G_URI_FLAGS_NONE, NULL);
+  if (!b_uri || !g_uri_get_host (b_uri))
+    return FALSE;
 
-  soup_uri_free (a_uri);
+  if (strcmp (g_uri_get_scheme (a_uri), g_uri_get_scheme (b_uri)) != 0)
+    return FALSE;
 
-  return retval;
+  if (g_uri_get_port (a_uri) != g_uri_get_port (b_uri))
+    return FALSE;
+
+  return g_ascii_strcasecmp (g_uri_get_host (a_uri), g_uri_get_host (b_uri)) == 0;
 }
 
 gboolean
 ephy_web_application_is_uri_allowed (const char *uri)
 {
-  EphyWebApplication *webapp = ephy_web_application_for_profile_directory (ephy_profile_dir ());
-  SoupURI *request_uri;
-  char **urls;
+  g_autoptr (EphyWebApplication) webapp = ephy_web_application_for_profile_directory (ephy_profile_dir ());
+  const char *scheme;
+  g_auto (GStrv) urls = NULL;
   guint i;
   gboolean matched = FALSE;
 
@@ -847,24 +828,22 @@ ephy_web_application_is_uri_allowed (const char *uri)
   if (g_strcmp0 (uri, "about:blank") == 0)
     return TRUE;
 
-  request_uri = soup_uri_new (uri);
-  if (!request_uri)
+  scheme = g_uri_peek_scheme (uri);
+  if (!scheme)
     return FALSE;
 
   urls = g_settings_get_strv (EPHY_SETTINGS_WEB_APP, EPHY_PREFS_WEB_APP_ADDITIONAL_URLS);
   for (i = 0; urls[i] && !matched; i++) {
     if (!strstr (urls[i], "://")) {
-      char *url = g_strdup_printf ("%s://%s", request_uri->scheme, urls[i]);
+      g_autofree char *url = NULL;
+
+      url = g_strdup_printf ("%s://%s", scheme, urls[i]);
 
       matched = g_str_has_prefix (uri, url);
-      g_free (url);
     } else {
       matched = g_str_has_prefix (uri, urls[i]);
     }
   }
-  g_strfreev (urls);
-
-  soup_uri_free (request_uri);
 
   return matched;
 }
@@ -874,30 +853,28 @@ ephy_web_icon_copy_cb (GFile        *file,
                        GAsyncResult *result,
                        gpointer      user_data)
 {
-  GError *error = NULL;
-  if (!g_file_copy_finish (file, result, &error)) {
+  g_autoptr (GError) error = NULL;
+  if (!g_file_copy_finish (file, result, &error))
     g_warning ("Failed to update web app icon: %s", error->message);
-    g_error_free (error);
-  }
 }
 
 gboolean
 ephy_web_application_save (EphyWebApplication *app)
 {
-  char *profile_dir;
-  char *desktop_file_path;
-  char *contents;
+  g_autofree char *profile_dir = NULL;
+  g_autofree char *desktop_file_path = NULL;
+  g_autofree char *contents = NULL;
   gboolean saved = FALSE;
-  GError *error = NULL;
+  g_autoptr (GError) error = NULL;
 
   profile_dir = ephy_web_application_get_profile_directory (app->id);
   desktop_file_path = g_build_filename (profile_dir, app->desktop_file, NULL);
   if (g_file_get_contents (desktop_file_path, &contents, NULL, &error)) {
-    GKeyFile *key;
-    char *name;
-    char *icon;
-    char *exec;
-    char **strings;
+    g_autoptr (GKeyFile) key = NULL;
+    g_autofree char *name = NULL;
+    g_autofree char *icon = NULL;
+    g_autofree char *exec = NULL;
+    g_auto (GStrv) strings = NULL;
     guint exec_length;
     gboolean changed = FALSE;
 
@@ -909,12 +886,11 @@ ephy_web_application_save (EphyWebApplication *app)
       changed = TRUE;
       g_key_file_set_string (key, "Desktop Entry", "Name", app->name);
     }
-    g_free (name);
 
     icon = g_key_file_get_string (key, "Desktop Entry", "Icon", NULL);
     if (g_strcmp0 (icon, app->icon_url) != 0) {
-      GFile *new_icon;
-      GFile *old_icon;
+      g_autoptr (GFile) new_icon = NULL;
+      g_autoptr (GFile) old_icon = NULL;
       changed = TRUE;
       new_icon = g_file_new_for_path (app->icon_url);
       old_icon = g_file_new_for_path (icon);
@@ -922,39 +898,45 @@ ephy_web_application_save (EphyWebApplication *app)
                          G_PRIORITY_DEFAULT, NULL, NULL, NULL,
                          (GAsyncReadyCallback)ephy_web_icon_copy_cb, NULL);
     }
-    g_free (icon);
 
     exec = g_key_file_get_string (key, "Desktop Entry", "Exec", NULL);
     strings = g_strsplit (exec, " ", -1);
-    g_free (exec);
 
     exec_length = g_strv_length (strings);
     if (g_strcmp0 (strings[exec_length - 1], app->url) != 0) {
       changed = TRUE;
       g_free (strings[exec_length - 1]);
       strings[exec_length - 1] = g_strdup (app->url);
+      g_free (exec);
       exec = g_strjoinv (" ", strings);
       g_key_file_set_string (key, "Desktop Entry", "Exec", exec);
-      g_free (exec);
     }
-    g_strfreev (strings);
 
     if (changed) {
       saved = g_key_file_save_to_file (key, desktop_file_path, &error);
-      if (!saved) {
+      if (!saved)
         g_warning ("Failed to save desktop file of web application: %s\n", error->message);
-        g_error_free (error);
-      }
     }
-    g_free (contents);
-    g_key_file_free (key);
   } else {
     g_warning ("Failed to load desktop file of web application: %s\n", error->message);
-    g_error_free (error);
   }
 
-  g_free (desktop_file_path);
-  g_free (profile_dir);
-
   return saved;
 }
+
+gboolean
+ephy_web_application_is_system (EphyWebApplication *app)
+{
+  GSettings *web_app_settings;
+  g_autofree char *profile_directory = NULL;
+  g_autofree char *name = NULL;
+  g_autofree char *path = NULL;
+
+  profile_directory = ephy_web_application_get_profile_directory (app->id);
+  name = g_path_get_basename (profile_directory);
+
+  path = g_build_path ("/", "/org/gnome/epiphany/web-apps/", name, "webapp/", NULL);
+  web_app_settings = g_settings_new_with_path (EPHY_PREFS_WEB_APP_SCHEMA, path);
+
+  return g_settings_get_boolean (web_app_settings, EPHY_PREFS_WEB_APP_SYSTEM);
+}
