@@ -35,16 +35,27 @@ typedef struct {
   char install_date[128];
 } EphyWebApplication;
 
+/**
+ * EphyWebApplicationOptions:
+ * @EPHY_WEB_APPLICATION_NONE: A default web application installed in the
+ *    user's home directory.
+ * @EPHY_WEB_APPLICATION_MOBILE_CAPABLE: Set when the meta tag
+ *    "apple-mobile-web-app-capable" is set. Causes back/forward navigation
+ *    buttons to be hidden.
+ * @EPHY_WEB_APPLICATION_SYSTEM: Set when the web application was installed
+ *    under /usr via deb/rpm rather than in the user's home directory.
+ */
 typedef enum {
   EPHY_WEB_APPLICATION_NONE,
   EPHY_WEB_APPLICATION_MOBILE_CAPABLE,
+  EPHY_WEB_APPLICATION_SYSTEM,
 } EphyWebApplicationOptions;
 
 #define EPHY_WEB_APP_ICON_NAME "app-icon.png"
 
 char               *ephy_web_application_get_app_id_from_name (const char *name);
 
-const char         *ephy_web_application_get_program_name_from_profile_directory (const char *profile_dir);
+const char         *ephy_web_application_get_gapplication_id_from_profile_directory (const char *profile_dir);
 
 char               *ephy_web_application_create (const char *id, const char *address, const char *name, GdkPixbuf *icon, EphyWebApplicationOptions options);
 
@@ -76,4 +87,8 @@ gboolean            ephy_web_application_is_uri_allowed (const char *uri);
 
 gboolean            ephy_web_application_save (EphyWebApplication *app);
 
+gboolean            ephy_web_application_is_system (EphyWebApplication *app);
+
+G_DEFINE_AUTOPTR_CLEANUP_FUNC (EphyWebApplication, ephy_web_application_free)
+
 G_END_DECLS
