@@ -53,8 +53,9 @@
  */
 
 struct _EphyLocationEntry {
-  GtkOverlay parent_instance;
+  GtkBin parent_instance;
 
+  GtkWidget *overlay;
   GtkWidget *url_entry;
   GtkWidget *page_action_box;
   GtkWidget *bookmark_icon;
@@ -114,7 +115,7 @@ static void ephy_location_entry_title_widget_interface_init (EphyTitleWidgetInte
 static void schedule_dns_prefetch (EphyLocationEntry *entry,
                                    const gchar       *url);
 
-G_DEFINE_TYPE_WITH_CODE (EphyLocationEntry, ephy_location_entry, GTK_TYPE_OVERLAY,
+G_DEFINE_TYPE_WITH_CODE (EphyLocationEntry, ephy_location_entry, GTK_TYPE_BIN,
                          G_IMPLEMENT_INTERFACE (EPHY_TYPE_TITLE_WIDGET,
                                                 ephy_location_entry_title_widget_interface_init))
 static gboolean
@@ -1062,6 +1063,11 @@ ephy_location_entry_construct_contents (EphyLocationEntry *entry)
 
   LOG ("EphyLocationEntry constructing contents %p", entry);
 
+  /* Overlay */
+  entry->overlay = gtk_overlay_new ();
+  gtk_widget_show (GTK_WIDGET (entry->overlay));
+  gtk_container_add (GTK_CONTAINER (entry), entry->overlay);
+
   /* URL entry */
   entry->url_entry = dzl_suggestion_entry_new ();
   dzl_suggestion_entry_set_compact (DZL_SUGGESTION_ENTRY (entry->url_entry), TRUE);
@@ -1081,7 +1087,7 @@ ephy_location_entry_construct_contents (EphyLocationEntry *entry)
   g_signal_connect (G_OBJECT (entry->url_entry), "changed", G_CALLBACK (editable_changed_cb), entry);
   g_signal_connect (G_OBJECT (entry->url_entry), "suggestion-selected", G_CALLBACK (suggestion_selected), entry);
   gtk_widget_show (GTK_WIDGET (entry->url_entry));
-  gtk_overlay_add_overlay (GTK_OVERLAY (entry), GTK_WIDGET (entry->url_entry));
+  gtk_container_add (GTK_CONTAINER (entry->overlay), GTK_WIDGET (entry->url_entry));
 
   /* Custom hover state. FIXME: Remove this for GTK4 */
   gtk_widget_add_events (GTK_WIDGET (entry->url_entry), GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK);
@@ -1093,7 +1099,7 @@ ephy_location_entry_construct_contents (EphyLocationEntry *entry)
   gtk_widget_set_halign (event, GTK_ALIGN_END);
   gtk_widget_show (event);
   g_signal_connect (G_OBJECT (event), "button-press-event", G_CALLBACK (event_button_press_event_cb), entry);
-  gtk_overlay_add_overlay (GTK_OVERLAY (entry), event);
+  gtk_overlay_add_overlay (GTK_OVERLAY (entry->overlay), event);
 
   /* Button Box */
   box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
@@ -1105,18 +1111,30 @@ ephy_location_entry_construct_contents (EphyLocationEntry *entry)
 
   /* Page action box */
   entry->page_action_box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
-  gtk_box_pack_start (GTK_BOX (box), entry->page_action_box, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (box), entry->page_action_box, FALSE, TRUE, 0);
 
   context = gtk_widget_get_style_context (box);
   gtk_style_context_add_class (context, "entry_icon_box");
 
+  /* Reader Mode */
+  entry->reader_mode_button = gtk_button_new_from_icon_name (NULL, GTK_ICON_SIZE_MENU);
+  gtk_widget_set_tooltip_text (entry->reader_mode_button, _("Toggle reader mode"));
+  entry->reader_mode_icon = gtk_button_get_image (GTK_BUTTON (entry->reader_mode_button));
+  gtk_box_pack_start (GTK_BOX (box), entry->reader_mode_button, FALSE, TRUE, 0);
+
+  context = gtk_widget_get_style_context (entry->reader_mode_icon);
+  gtk_style_context_add_class (context, "entry_icon");
+
+  update_reader_icon (entry);
+  g_signal_connect_object (gtk_settings_get_default (), "notify::gtk-icon-theme-name",
+                           G_CALLBACK (update_reader_icon), entry, G_CONNECT_SWAPPED);
+
   /* Bookmark */
   entry->bookmark_button = gtk_button_new_from_icon_name ("non-starred-symbolic", GTK_ICON_SIZE_MENU);
   gtk_widget_set_tooltip_text (entry->bookmark_button, _("Bookmark this page"));
   entry->bookmark_icon = gtk_button_get_image (GTK_BUTTON (entry->bookmark_button));
-  gtk_widget_set_valign (entry->bookmark_icon, GTK_ALIGN_CENTER);
   g_signal_connect (G_OBJECT (entry->bookmark_button), "clicked", G_CALLBACK (bookmark_icon_button_clicked_cb), entry);
-  gtk_box_pack_end (GTK_BOX (box), entry->bookmark_button, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (box), entry->bookmark_button, FALSE, TRUE, 0);
 
   context = gtk_widget_get_style_context (entry->bookmark_icon);
   gtk_style_context_add_class (context, "entry_icon");
@@ -1127,20 +1145,6 @@ ephy_location_entry_construct_contents (EphyLocationEntry *entry)
                    "visible",
                    G_SETTINGS_BIND_GET | G_SETTINGS_BIND_INVERT_BOOLEAN);
 
-  /* Reader Mode */
-  entry->reader_mode_button = gtk_button_new_from_icon_name (NULL, GTK_ICON_SIZE_MENU);
-  gtk_widget_set_tooltip_text (entry->reader_mode_button, _("Toggle reader mode"));
-  entry->reader_mode_icon = gtk_button_get_image (GTK_BUTTON (entry->reader_mode_button));
-  gtk_widget_set_valign (entry->reader_mode_icon, GTK_ALIGN_CENTER);
-  gtk_box_pack_end (GTK_BOX (box), entry->reader_mode_button, FALSE, FALSE, 0);
-
-  context = gtk_widget_get_style_context (entry->reader_mode_icon);
-  gtk_style_context_add_class (context, "entry_icon");
-
-  update_reader_icon (entry);
-  g_signal_connect_object (gtk_settings_get_default (), "notify::gtk-icon-theme-name",
-                           G_CALLBACK (update_reader_icon), entry, G_CONNECT_SWAPPED);
-
   g_object_connect (entry->url_entry,
                     "signal::icon-press", G_CALLBACK (icon_button_icon_press_event_cb), entry,
                     "signal::populate-popup", G_CALLBACK (entry_populate_popup_cb), entry,
@@ -1586,7 +1590,7 @@ void
 ephy_location_entry_page_action_add (EphyLocationEntry *entry,
                                      GtkWidget         *action)
 {
-  gtk_box_pack_end (GTK_BOX (entry->page_action_box), action, FALSE, FALSE, 0);
+  gtk_box_pack_end (GTK_BOX (entry->page_action_box), action, FALSE, TRUE, 0);
 
   gtk_widget_show (entry->page_action_box);
 }
