@@ -32,17 +32,16 @@
 #include <glib/gi18n.h>
 
 struct _EphyBookmarksPopover {
-  GtkPopover             parent_instance;
+  GtkPopover parent_instance;
 
-  GtkWidget             *toplevel_stack;
-  GtkWidget             *bookmarks_list_box;
-  GtkWidget             *tags_list_box;
-  GtkWidget             *tag_detail_list_box;
-  GtkWidget             *tag_detail_back_button;
-  GtkWidget             *tag_detail_label;
-  char                  *tag_detail_tag;
+  GtkWidget *toplevel_stack;
+  GtkWidget *bookmarks_list_box;
+  GtkWidget *tags_list_box;
+  GtkWidget *tag_detail_list_box;
+  GtkWidget *tag_detail_label;
+  char *tag_detail_tag;
 
-  EphyBookmarksManager  *manager;
+  EphyBookmarksManager *manager;
 };
 
 G_DEFINE_TYPE (EphyBookmarksPopover, ephy_bookmarks_popover, GTK_TYPE_POPOVER)
@@ -50,7 +49,7 @@ G_DEFINE_TYPE (EphyBookmarksPopover, ephy_bookmarks_popover, GTK_TYPE_POPOVER)
 #define EPHY_LIST_BOX_ROW_TYPE_BOOKMARK "bookmark"
 #define EPHY_LIST_BOX_ROW_TYPE_TAG "tag"
 
-static GtkWidget *create_bookmark_row (gpointer item, gpointer user_data);
+static GtkWidget * create_bookmark_row (gpointer item, gpointer user_data);
 static GtkWidget *create_tag_row (const char *tag);
 
 static void
@@ -178,7 +177,7 @@ ephy_bookmarks_popover_bookmark_tag_removed_cb (EphyBookmarksPopover *self,
                                         ephy_bookmark_get_url (bookmark));
 
     /* If we removed the tag's last bookmark, switch back to the tags list. */
-    if (g_sequence_is_empty (ephy_bookmarks_manager_get_bookmarks_with_tag (self->manager, tag))) {
+    if (ephy_bookmarks_manager_has_bookmarks_with_tag (self->manager, tag)) {
       GActionGroup *group;
       GAction *action;
 
@@ -189,14 +188,14 @@ ephy_bookmarks_popover_bookmark_tag_removed_cb (EphyBookmarksPopover *self,
   }
 
   /* If the tag no longer contains bookmarks, remove it from the tags list */
-  if (g_sequence_is_empty (ephy_bookmarks_manager_get_bookmarks_with_tag (self->manager, tag))) {
-      children = gtk_container_get_children (GTK_CONTAINER (self->tags_list_box));
-      for (l = children; l != NULL; l = l->next) {
-        title = g_object_get_data (G_OBJECT (l->data), "title");
-        if (g_strcmp0 (title, tag) == 0)
-          gtk_container_remove (GTK_CONTAINER (self->tags_list_box), GTK_WIDGET (l->data));
-      }
-      g_list_free (children);
+  if (ephy_bookmarks_manager_has_bookmarks_with_tag (self->manager, tag)) {
+    children = gtk_container_get_children (GTK_CONTAINER (self->tags_list_box));
+    for (l = children; l != NULL; l = l->next) {
+      title = g_object_get_data (G_OBJECT (l->data), "title");
+      if (g_strcmp0 (title, tag) == 0)
+        gtk_container_remove (GTK_CONTAINER (self->tags_list_box), GTK_WIDGET (l->data));
+    }
+    g_list_free (children);
   }
 }
 
@@ -236,18 +235,20 @@ create_tag_row (const char *tag)
   g_object_set (G_OBJECT (row), "height-request", 40, NULL);
 
   box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
-  gtk_widget_set_halign (box, GTK_ALIGN_START);
 
   if (g_strcmp0 (tag, EPHY_BOOKMARKS_FAVORITES_TAG) == 0) {
     image = gtk_image_new_from_icon_name ("emblem-favorite-symbolic", GTK_ICON_SIZE_MENU);
-    label = gtk_label_new (_(EPHY_BOOKMARKS_FAVORITES_TAG));
   } else {
     image = gtk_image_new_from_icon_name ("ephy-bookmark-tag-symbolic", GTK_ICON_SIZE_MENU);
-    label = gtk_label_new (tag);
   }
+  label = gtk_label_new (tag);
 
-  gtk_box_pack_start (GTK_BOX (box), image, FALSE, FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (box), label, TRUE, FALSE, 0);
+  gtk_widget_set_hexpand (label, TRUE);
+  gtk_label_set_xalign (GTK_LABEL (label), 0);
+  gtk_label_set_ellipsize (GTK_LABEL (label), PANGO_ELLIPSIZE_END);
+
+  gtk_box_pack_start (GTK_BOX (box), image, FALSE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX (box), label, FALSE, TRUE, 0);
 
   gtk_container_add (GTK_CONTAINER (row), box);
   gtk_widget_show_all (row);
@@ -292,7 +293,7 @@ ephy_bookmarks_popover_bookmark_removed_cb (EphyBookmarksPopover *self,
   if (g_list_model_get_n_items (G_LIST_MODEL (self->manager)) == 0) {
     gtk_stack_set_visible_child_name (GTK_STACK (self->toplevel_stack), "empty-state");
   } else if (g_strcmp0 (gtk_stack_get_visible_child_name (GTK_STACK (self->toplevel_stack)), "tag_detail") == 0 &&
-             g_sequence_is_empty (ephy_bookmarks_manager_get_bookmarks_with_tag (self->manager, self->tag_detail_tag))) {
+             ephy_bookmarks_manager_has_bookmarks_with_tag (self->manager, self->tag_detail_tag)) {
     /* If we removed the tag's last bookmark, switch back to the tags list. */
     GActionGroup *group;
     GAction *action;
@@ -345,7 +346,8 @@ ephy_bookmarks_popover_tag_deleted_cb (EphyBookmarksPopover *self,
 }
 
 static int
-tags_list_box_sort_func (GtkListBoxRow *row1, GtkListBoxRow *row2)
+tags_list_box_sort_func (GtkListBoxRow *row1,
+                         GtkListBoxRow *row2)
 {
   const char *type1;
   const char *type2;
@@ -363,7 +365,7 @@ tags_list_box_sort_func (GtkListBoxRow *row1, GtkListBoxRow *row2)
 
   if (g_strcmp0 (type1, EPHY_LIST_BOX_ROW_TYPE_TAG) == 0
       && g_strcmp0 (type2, EPHY_LIST_BOX_ROW_TYPE_TAG) == 0)
-      return ephy_bookmark_tags_compare (title1, title2);
+    return ephy_bookmark_tags_compare (title1, title2);
 
   if (g_strcmp0 (type1, EPHY_LIST_BOX_ROW_TYPE_TAG) == 0)
     return -1;
@@ -410,10 +412,7 @@ ephy_bookmarks_popover_show_tag_detail (EphyBookmarksPopover *self,
     gtk_container_add (GTK_CONTAINER (self->tag_detail_list_box), row);
   }
 
-  if (strcmp (tag, EPHY_BOOKMARKS_FAVORITES_TAG) == 0)
-    gtk_label_set_label (GTK_LABEL (self->tag_detail_label), _(EPHY_BOOKMARKS_FAVORITES_TAG));
-  else
-    gtk_label_set_label (GTK_LABEL (self->tag_detail_label), tag);
+  gtk_label_set_label (GTK_LABEL (self->tag_detail_label), tag);
 
   gtk_stack_set_visible_child_name (GTK_STACK (self->toplevel_stack), "tag_detail");
 
@@ -425,9 +424,29 @@ ephy_bookmarks_popover_show_tag_detail (EphyBookmarksPopover *self,
 }
 
 static void
-ephy_bookmarks_popover_list_box_row_activated_cb (EphyBookmarksPopover   *self,
-                                                  GtkListBoxRow          *row,
-                                                  GtkListBox             *box)
+ephy_bookmarks_popover_open_bookmark (EphyBookmarksPopover *self,
+                                      GtkListBoxRow        *row)
+{
+  GtkWidget *window;
+  GActionGroup *action_group;
+  GAction *action;
+  const char *url;
+
+  window = gtk_widget_get_ancestor (GTK_WIDGET (self), EPHY_TYPE_WINDOW);
+  g_assert (EPHY_IS_WINDOW (window));
+  action_group = gtk_widget_get_action_group (window, "win");
+  g_assert (action_group != NULL);
+  action = g_action_map_lookup_action (G_ACTION_MAP (action_group), "open-bookmark");
+  g_assert (action != NULL);
+  url = ephy_bookmark_row_get_bookmark_url (EPHY_BOOKMARK_ROW (row));
+
+  g_action_activate (action, g_variant_new_string (url));
+}
+
+static void
+ephy_bookmarks_popover_list_box_row_activated_cb (EphyBookmarksPopover *self,
+                                                  GtkListBoxRow        *row,
+                                                  GtkListBox           *box)
 {
   const char *type;
   const char *tag;
@@ -438,26 +457,29 @@ ephy_bookmarks_popover_list_box_row_activated_cb (EphyBookmarksPopover   *self,
 
   type = g_object_get_data (G_OBJECT (row), "type");
   if (g_strcmp0 (type, EPHY_LIST_BOX_ROW_TYPE_BOOKMARK) == 0) {
-    GtkWidget *window;
-    GActionGroup *action_group;
-    GAction *action;
-    const char *url;
-
-    window = gtk_widget_get_ancestor (GTK_WIDGET (self), EPHY_TYPE_WINDOW);
-    g_assert (EPHY_IS_WINDOW (window));
-    action_group = gtk_widget_get_action_group (window, "win");
-    g_assert (action_group != NULL);
-    action = g_action_map_lookup_action (G_ACTION_MAP (action_group), "open-bookmark");
-    g_assert (action != NULL);
-    url = ephy_bookmark_row_get_bookmark_url (EPHY_BOOKMARK_ROW (row));
-
-    g_action_activate (action, g_variant_new_string (url));
+    ephy_bookmarks_popover_open_bookmark (self, row);
   } else {
     tag = g_object_get_data (G_OBJECT (row), "title");
     ephy_bookmarks_popover_show_tag_detail (self, tag);
   }
 }
 
+static gboolean
+ephy_bookmarks_popover_list_box_button_release_event_cb (EphyBookmarksPopover *self,
+                                                         GdkEvent             *event,
+                                                         GtkListBox           *box)
+{
+  GdkEventButton *event_button = (GdkEventButton *)event;
+  GtkListBoxRow *row = gtk_list_box_get_row_at_y (box, event_button->y);
+
+  if (event_button->button != GDK_BUTTON_MIDDLE)
+    return GDK_EVENT_PROPAGATE;
+
+  ephy_bookmarks_popover_open_bookmark (self, row);
+
+  return GDK_EVENT_STOP;
+}
+
 static void
 ephy_bookmarks_popover_finalize (GObject *object)
 {
@@ -481,7 +503,6 @@ ephy_bookmarks_popover_class_init (EphyBookmarksPopoverClass *klass)
   gtk_widget_class_bind_template_child (widget_class, EphyBookmarksPopover, bookmarks_list_box);
   gtk_widget_class_bind_template_child (widget_class, EphyBookmarksPopover, tags_list_box);
   gtk_widget_class_bind_template_child (widget_class, EphyBookmarksPopover, tag_detail_list_box);
-  gtk_widget_class_bind_template_child (widget_class, EphyBookmarksPopover, tag_detail_back_button);
   gtk_widget_class_bind_template_child (widget_class, EphyBookmarksPopover, tag_detail_label);
 }
 
@@ -493,9 +514,9 @@ static void
 ephy_bookmarks_popover_init (EphyBookmarksPopover *self)
 {
   GSequence *tags;
-  GSequence *bookmarks;
   GSequenceIter *iter;
-  GSimpleActionGroup *group;
+  g_autoptr (GSequence) bookmarks = NULL;
+  g_autoptr (GSimpleActionGroup) group = NULL;
 
   gtk_widget_init_template (GTK_WIDGET (self));
 
@@ -506,7 +527,6 @@ ephy_bookmarks_popover_init (EphyBookmarksPopover *self)
                                    G_N_ELEMENTS (entries), self);
   gtk_widget_insert_action_group (GTK_WIDGET (self), "popover",
                                   G_ACTION_GROUP (group));
-  g_object_unref (group);
 
   gtk_list_box_bind_model (GTK_LIST_BOX (self->bookmarks_list_box),
                            G_LIST_MODEL (self->manager),
@@ -530,7 +550,7 @@ ephy_bookmarks_popover_init (EphyBookmarksPopover *self)
     const char *tag = g_sequence_get (iter);
     GtkWidget *tag_row;
 
-    if (!g_sequence_is_empty (ephy_bookmarks_manager_get_bookmarks_with_tag (self->manager, tag))) {
+    if (!ephy_bookmarks_manager_has_bookmarks_with_tag (self->manager, tag)) {
       tag_row = create_tag_row (tag);
       gtk_container_add (GTK_CONTAINER (self->tags_list_box), tag_row);
     }
@@ -576,6 +596,9 @@ ephy_bookmarks_popover_init (EphyBookmarksPopover *self)
   g_signal_connect_object (self->tag_detail_list_box, "row-activated",
                            G_CALLBACK (ephy_bookmarks_popover_list_box_row_activated_cb),
                            self, G_CONNECT_SWAPPED);
+  g_signal_connect_object (self->bookmarks_list_box, "button-release-event",
+                           G_CALLBACK (ephy_bookmarks_popover_list_box_button_release_event_cb),
+                           self, G_CONNECT_SWAPPED);
 }
 
 EphyBookmarksPopover *
