@@ -20,30 +20,36 @@
  */
 
 #include "ephy-action-bar.h"
+#include "ephy-add-bookmark-popover.h"
+#include "ephy-location-entry.h"
 #include "ephy-pages-button.h"
-#include "ephy-pages-popover.h"
 #include "ephy-settings.h"
 #include "ephy-shell.h"
+#include "ephy-tab-view.h"
 
 enum {
   PROP_0,
   PROP_WINDOW,
+  PROP_CAN_REVEAL,
   N_PROPERTIES
 };
 
 static GParamSpec *object_properties[N_PROPERTIES] = { NULL, };
 
 struct _EphyActionBar {
-  GtkRevealer parent_instance;
+  GtkBin parent_instance;
 
   EphyWindow *window;
+  GtkRevealer *revealer;
   EphyActionBarStart *action_bar_start;
   EphyActionBarEnd *action_bar_end;
   EphyPagesButton *pages_button;
-  GtkNotebook *notebook;
+
+  EphyAdaptiveMode adaptive_mode;
+  gboolean can_reveal;
 };
 
-G_DEFINE_TYPE (EphyActionBar, ephy_action_bar, GTK_TYPE_REVEALER)
+G_DEFINE_TYPE (EphyActionBar, ephy_action_bar, GTK_TYPE_BIN)
 
 static void
 sync_chromes_visibility (EphyActionBar *action_bar)
@@ -59,12 +65,15 @@ sync_chromes_visibility (EphyActionBar *action_bar)
 }
 
 static void
-update_pages_button (EphyActionBar *action_bar)
+update_revealer (EphyActionBar *action_bar)
 {
-  int n_pages;
+  gboolean reveal = action_bar->can_reveal &&
+                    (action_bar->adaptive_mode == EPHY_ADAPTIVE_MODE_NARROW);
 
-  n_pages = gtk_notebook_get_n_pages (action_bar->notebook);
-  ephy_pages_button_set_n_pages (action_bar->pages_button, n_pages);
+  if (reveal)
+    gtk_widget_show (GTK_WIDGET (action_bar));
+
+  gtk_revealer_set_reveal_child (action_bar->revealer, reveal);
 }
 
 static void
@@ -80,6 +89,10 @@ ephy_action_bar_set_property (GObject      *object,
       action_bar->window = EPHY_WINDOW (g_value_get_object (value));
       g_object_notify_by_pspec (object, object_properties[PROP_WINDOW]);
       break;
+    case PROP_CAN_REVEAL:
+      action_bar->can_reveal = g_value_get_boolean (value);
+      update_revealer (action_bar);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
   }
@@ -97,6 +110,9 @@ ephy_action_bar_get_property (GObject    *object,
     case PROP_WINDOW:
       g_value_set_object (value, action_bar->window);
       break;
+    case PROP_CAN_REVEAL:
+      g_value_set_boolean (value, action_bar->can_reveal);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
   }
@@ -106,21 +122,19 @@ static void
 ephy_action_bar_constructed (GObject *object)
 {
   EphyActionBar *action_bar = EPHY_ACTION_BAR (object);
+  EphyTabView *view;
 
   G_OBJECT_CLASS (ephy_action_bar_parent_class)->constructed (object);
 
-  action_bar->notebook = GTK_NOTEBOOK (ephy_window_get_notebook (action_bar->window));
-  update_pages_button (action_bar);
+  view = ephy_window_get_tab_view (action_bar->window);
 
   g_signal_connect_object (action_bar->window, "notify::chrome",
                            G_CALLBACK (sync_chromes_visibility), action_bar,
                            G_CONNECT_SWAPPED);
-  g_signal_connect_object (action_bar->notebook, "page-added",
-                           G_CALLBACK (update_pages_button), action_bar,
-                           G_CONNECT_SWAPPED);
-  g_signal_connect_object (action_bar->notebook, "page-removed",
-                           G_CALLBACK (update_pages_button), action_bar,
-                           G_CONNECT_SWAPPED);
+
+  g_object_bind_property (view, "n-pages",
+                          action_bar->pages_button, "n-pages",
+                          G_BINDING_SYNC_CREATE);
 }
 
 static void
@@ -140,6 +154,13 @@ ephy_action_bar_class_init (EphyActionBarClass *klass)
                          EPHY_TYPE_WINDOW,
                          G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS);
 
+  object_properties[PROP_CAN_REVEAL] =
+    g_param_spec_boolean ("can-reveal",
+                          "Can Reveal",
+                          "Whether the action bar can be revealed",
+                          TRUE,
+                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+
   g_object_class_install_properties (gobject_class,
                                      N_PROPERTIES,
                                      object_properties);
@@ -147,6 +168,9 @@ ephy_action_bar_class_init (EphyActionBarClass *klass)
   gtk_widget_class_set_template_from_resource (widget_class,
                                                "/org/gnome/epiphany/gtk/action-bar.ui");
 
+  gtk_widget_class_bind_template_child (widget_class,
+                                        EphyActionBar,
+                                        revealer);
   gtk_widget_class_bind_template_child (widget_class,
                                         EphyActionBar,
                                         action_bar_start);
@@ -173,6 +197,16 @@ ephy_action_bar_init (EphyActionBar *action_bar)
   mode = ephy_embed_shell_get_mode (EPHY_EMBED_SHELL (ephy_shell_get_default ()));
   gtk_widget_set_visible (GTK_WIDGET (action_bar->pages_button),
                           mode != EPHY_EMBED_SHELL_MODE_APPLICATION);
+
+  ephy_action_bar_start_set_adaptive_mode (action_bar->action_bar_start,
+                                           EPHY_ADAPTIVE_MODE_NARROW);
+
+  g_object_bind_property (action_bar->revealer, "child-revealed",
+                          action_bar, "visible",
+                          G_BINDING_DEFAULT);
+
+  action_bar->can_reveal = true;
+  update_revealer (action_bar);
 }
 
 EphyActionBar *
@@ -199,14 +233,9 @@ void
 ephy_action_bar_set_adaptive_mode (EphyActionBar    *action_bar,
                                    EphyAdaptiveMode  adaptive_mode)
 {
-  switch (adaptive_mode) {
-    case EPHY_ADAPTIVE_MODE_NORMAL:
-      gtk_revealer_set_reveal_child (GTK_REVEALER (action_bar), FALSE);
+  action_bar->adaptive_mode = adaptive_mode;
+  ephy_action_bar_end_set_show_bookmark_button (action_bar->action_bar_end,
+                                                adaptive_mode == EPHY_ADAPTIVE_MODE_NARROW);
 
-      break;
-    case EPHY_ADAPTIVE_MODE_NARROW:
-      gtk_revealer_set_reveal_child (GTK_REVEALER (action_bar), TRUE);
-
-      break;
-  }
+  update_revealer (action_bar);
 }
